#########################################################
## This program is a simple pixel editor
## The pixels are stored in a grid (a list of lists)
## and every pixel an only be black or white.
## Clicking on one of the pixels inverts its color.
## The GUI has also 3 icons:
## - load -> loads the pixels from a JSON file (if it exists)
## - save -> saves the pixels to a JSON file
## - quit -> closes the program
##
## A mini version of the pixels is drawn on the top-left corner 
## of the drawing window.
#########################################################

from medialib import *
import json
initialize()

pixels = [ [0,0,0,0,0,0,0,0,0,0],
           [0,0,1,1,0,0,1,1,0,0],
           [0,0,0,1,0,0,0,1,0,0],
           [0,0,0,0,0,0,0,0,0,0],
           [0,0,0,1,0,0,1,0,0,0],
           [0,0,0,0,1,1,0,0,0,0],
           [0,0,0,0,0,0,0,0,0,0]
         ]

color_palette = [ [50,50,50] , [200,200,255] ]
size = 50
icon_clicked = -1 ## no icon has been clicked so far

def draw_pixels(pix_grid):
    global icon_clicked
    has_changed = False
    clear()

    ## icons
    if icon_clicked==1:
        draw('load.png',650,150-50)
        has_changed = True
    else:
        draw('load.png',650,150)
    if icon_clicked==2:
        draw('save.png',650,300-50)
        has_changed = True
    else:
        draw('save.png',650,300)

    draw('quit.png',650,500)

    ## draw all pixels, line by line
    y = 0
    for line in pix_grid:
        y+=1
        x = 0
        for pixel in line:
            x += 1
            rect(x*size,y*size,
                size-2,size-2,
                color_palette[ pixel ][0],
                color_palette[ pixel ][1],
                color_palette[ pixel ][2]
            )

    icon_clicked = -1 ## reset which icon you clicked

def draw_mini_pixels(pix_grid):
    k = 2 ## every pixel is drawn as a 2x2 square
    y = 0
    for line in pix_grid:
        y+=1
        x = 0
        for pixel in line:
            x += 1
            rect(x*k+10,y*k+10,
                k,k,
                color_palette[ pixel ][0],
                color_palette[ pixel ][1],
                color_palette[ pixel ][2]
            )

is_running = True
has_changed = True
press_again_in = -1

while is_running:
    if press_again_in>-1:
        press_again_in -= 1

    if has_changed:
        draw_pixels(pixels)
        draw_mini_pixels(pixels)

    mx,my = get_mouse_pos()
    if is_mouse_press():
        if point_inside_rect(mx,my, 650,500,86,94):
            ## clicked on quit button
            is_running = False
        elif point_inside_rect(mx,my, 650,150,93,100):
            ## clicked on LOAD button
            icon_clicked = 1
            try:
                the_file = open('pixels_saved.json','r')
                pixels = json.load( the_file )
                the_file.close()
                has_changed = True
            except:
                pass ## do nothing if the file does not exist
        elif point_inside_rect(mx,my, 650,300,93,100):
            ## clicked on SAVE button
            icon_clicked = 2
            the_file = open('pixels_saved.json','w')
            json.dump( pixels , the_file, indent=3)
            the_file.close()
        else:
            if press_again_in==-1:
                gridX = int(mx/size)
                gridY = int(my/size)
                ## debug only: print(gridX,gridY)
                if gridX>=1 and gridX<=10 and gridY>=1 and gridY<=7:
                    pixels[gridY-1][gridX-1] = 1-pixels[gridY-1][gridX-1]
                    press_again_in = 5
                    has_changed = True

    wait(0.050) ## to slow down a bit the loop

all_done()
